@@ -3,41 +3,59 @@ use std::fmt::Display;
 use quote::{quote, ToTokens};
 
 /// Tokenizes OpenAPI data type correctly according to the Rust type
-pub(crate) struct ComponentType<'a, T: Display>(pub &'a T);
+pub struct ComponentType<'a, T: Display>(pub &'a T);
 
 impl<'a, T> ComponentType<'a, T>
 where
     T: Display,
 {
     /// Check whether type is known to be primitive in wich case returns true.
-    pub(crate) fn is_primitive(&self) -> bool {
+    pub fn is_primitive(&self) -> bool {
         let name = &*self.0.to_string();
 
-        let primitive = is_primitive(name);
+        #[cfg(not(any(
+            feature = "chrono",
+            feature = "chrono_with_format",
+            feature = "decimal",
+            feature = "rocket_extras",
+            feature = "uuid"
+        )))]
+        {
+            is_primitive(name)
+        }
 
         #[cfg(any(
-            feature = "chrono_types",
-            feature = "chrono_types_with_format",
-            feature = "rocket_extras"
+            feature = "chrono",
+            feature = "chrono_with_format",
+            feature = "decimal",
+            feature = "rocket_extras",
+            feature = "uuid",
         ))]
-        let mut primitive = primitive;
+        {
+            let mut primitive = is_primitive(name);
 
-        #[cfg(any(feature = "chrono_types", feature = "chrono_types_with_format"))]
-        if !primitive {
-            primitive = is_primitive_chrono(name);
+            #[cfg(any(feature = "chrono", feature = "chrono_with_format"))]
+            if !primitive {
+                primitive = is_primitive_chrono(name);
+            }
+
+            #[cfg(feature = "decimal")]
+            if !primitive {
+                primitive = is_primitive_rust_decimal(name);
+            }
+
+            #[cfg(feature = "rocket_extras")]
+            if !primitive {
+                primitive = matches!(name, "PathBuf");
+            }
+
+            #[cfg(feature = "uuid")]
+            if !primitive {
+                primitive = matches!(name, "Uuid");
+            }
+
+            primitive
         }
-
-        #[cfg(feature = "decimal")]
-        if !primitive {
-            primitive = is_primitive_rust_decimal(name);
-        }
-
-        #[cfg(feature = "rocket_extras")]
-        if !primitive {
-            primitive = matches!(name, "PathBuf");
-        }
-
-        primitive
     }
 }
 
@@ -67,13 +85,13 @@ fn is_primitive(name: &str) -> bool {
 }
 
 #[inline]
-#[cfg(any(feature = "chrono_types", feature = "chrono_types_with_format"))]
+#[cfg(any(feature = "chrono", feature = "chrono_with_format"))]
 fn is_primitive_chrono(name: &str) -> bool {
     matches!(name, "DateTime" | "Date" | "Duration")
 }
 
 #[inline]
-#[cfg(feature = "chrono_types")]
+#[cfg(feature = "chrono")]
 fn is_primitive_rust_decimal(name: &str) -> bool {
     matches!(name, "Decimal")
 }
@@ -93,7 +111,7 @@ where
             "i8" | "i16" | "i32" | "i64" | "i128" | "isize" | "u8" | "u16" | "u32" | "u64"
             | "u128" | "usize" => tokens.extend(quote! { utoipa::openapi::ComponentType::Integer }),
             "f32" | "f64" => tokens.extend(quote! { utoipa::openapi::ComponentType::Number }),
-            #[cfg(any(feature = "chrono_types", feature = "chrono_types_with_format"))]
+            #[cfg(any(feature = "chrono", feature = "chrono_with_format"))]
             "DateTime" | "Date" | "Duration" => {
                 tokens.extend(quote! { utoipa::openapi::ComponentType::String })
             }
@@ -101,30 +119,42 @@ where
             "Decimal" => tokens.extend(quote! { utoipa::openapi::ComponentType::String }),
             #[cfg(feature = "rocket_extras")]
             "PathBuf" => tokens.extend(quote! { utoipa::openapi::ComponentType::String }),
+            #[cfg(feature = "uuid")]
+            "Uuid" => tokens.extend(quote! { utoipa::openapi::ComponentType::String }),
             _ => tokens.extend(quote! { utoipa::openapi::ComponentType::Object }),
         }
     }
 }
 
 /// Tokenizes OpenAPI data type format correctly by given Rust type.
-pub(crate) struct ComponentFormat<T: Display>(pub(crate) T);
+pub struct ComponentFormat<T: Display>(pub(crate) T);
 
 impl<T: Display> ComponentFormat<T> {
     /// Check is the format know format. Known formats can be used within `quote! {...}` statements.
-    pub(crate) fn is_known_format(&self) -> bool {
+    pub fn is_known_format(&self) -> bool {
         let name = &*self.0.to_string();
 
-        let known_format = is_known_format(name);
-
-        #[cfg(feature = "chrono_types_with_format")]
-        let mut known_format = known_format;
-
-        #[cfg(feature = "chrono_types_with_format")]
-        if !known_format {
-            known_format = matches!(name, "DateTime" | "Date");
+        #[cfg(not(any(feature = "chrono_with_format", feature = "uuid")))]
+        {
+            is_known_format(name)
         }
 
-        known_format
+        #[cfg(any(feature = "chrono_with_format", feature = "uuid"))]
+        {
+            let mut known_format = is_known_format(name);
+
+            #[cfg(feature = "chrono_with_format")]
+            if !known_format {
+                known_format = matches!(name, "DateTime" | "Date");
+            }
+
+            #[cfg(feature = "uuid")]
+            if !known_format {
+                known_format = matches!(name, "Uuid");
+            }
+
+            known_format
+        }
     }
 }
 
@@ -146,10 +176,12 @@ impl<T: Display> ToTokens for ComponentFormat<T> {
             }
             "i64" | "u64" => tokens.extend(quote! { utoipa::openapi::ComponentFormat::Int64 }),
             "f32" | "f64" => tokens.extend(quote! { utoipa::openapi::ComponentFormat::Float }),
-            #[cfg(feature = "chrono_types_with_format")]
+            #[cfg(feature = "chrono_with_format")]
             "DateTime" => tokens.extend(quote! { utoipa::openapi::ComponentFormat::DateTime }),
-            #[cfg(feature = "chrono_types_with_format")]
+            #[cfg(feature = "chrono_with_format")]
             "Date" => tokens.extend(quote! { utoipa::openapi::ComponentFormat::Date }),
+            #[cfg(feature = "uuid")]
+            "Uuid" => tokens.extend(quote! { utoipa::openapi::ComponentFormat::Uuid }),
             _ => (),
         }
     }
