@@ -7,16 +7,18 @@
 #![warn(missing_docs)]
 #![warn(rustdoc::broken_intra_doc_links)]
 
-use std::{borrow::Cow, mem};
+use std::{borrow::Cow, mem, ops::Deref};
 
-use component::Component;
 use doc_comment::CommentAttributes;
+use schema::component::Component;
 
 use ext::{PathOperationResolver, PathOperations, PathResolver};
 use openapi::OpenApi;
 use proc_macro::TokenStream;
 use proc_macro_error::{proc_macro_error, OptionExt, ResultExt};
 use quote::{quote, ToTokens, TokenStreamExt};
+#[cfg(feature = "actix_extras")]
+use schema::into_params::IntoParams;
 
 use proc_macro2::{Group, Ident, Punct, TokenStream as TokenStream2};
 use syn::{
@@ -24,15 +26,15 @@ use syn::{
     parse::{Parse, ParseBuffer, ParseStream},
     punctuated::Punctuated,
     token::Bracket,
-    DeriveInput, ItemFn, Token,
+    DeriveInput, ExprPath, ItemFn, Lit, LitStr, Token,
 };
 
-mod component;
 mod component_type;
 mod doc_comment;
 mod ext;
 mod openapi;
 mod path;
+mod schema;
 mod security_requirement;
 
 use crate::path::{Path, PathAttr};
@@ -57,19 +59,19 @@ use ext::ArgumentResolver;
 /// `#[deprecated  = "There is better way to do this"]` the reason would not render in OpenAPI spec.
 ///
 /// # Struct Optional Configuration Options
-/// * `example = ...` Can be either `json!(...)` or literal string that can be parsed to json. `json!`
-///   should be something that `serde_json::json!` can parse as a `serde_json::Value`. [^json]
+/// * `example = ...` Can be either _`json!(...)`_ or literal string that can be parsed to json. _`json!`_
+///   should be something that _`serde_json::json!`_ can parse as a _`serde_json::Value`_. [^json]
 /// * `xml(...)` Can be used to define [`Xml`][xml] object properties applicable to Structs.
 ///  
-/// [^json]: **json** feature need to be enabled for `json!(...)` type to work.
+/// [^json]: **json** feature need to be enabled for _`json!(...)`_ type to work.
 ///
 /// # Enum Optional Configuration Options
-/// * `example = ...` Can be method reference or literal value. [^json2]
-/// * `default = ...` Can be method reference or literal value. [^json2]
+/// * `example = ...` Can be literal value, method reference or _`json!(...)`_. [^json2]
+/// * `default = ...` Can be literal value, method reference or _`json!(...)`_. [^json2]
 ///
 /// # Unnamed Field Struct Optional Configuration Options
-/// * `example = ...` Can be method reference or literal value. [^json2]
-/// * `default = ...` Can be method reference or literal value. [^json2]
+/// * `example = ...` Can be literal value, method reference or _`json!(...)`_. [^json2]
+/// * `default = ...` Can be literal value, method reference or _`json!(...)`_. [^json2]
 /// * `format = ...` [`ComponentFormat`][format] to use for the property. By default the format is derived from
 ///   the type of the property according OpenApi spec.
 /// * `value_type = ...` Can be used to override default type derived from type of the field used in OpenAPI spec.
@@ -78,8 +80,8 @@ use ext::ArgumentResolver;
 ///   type used to certain type. Value type may only be [`primitive`][primitive] type or [`String`]. Generic types are not allowed.
 ///
 /// # Named Fields Optional Configuration Options
-/// * `example = ...` Can be method reference or literal value. [^json2]
-/// * `default = ...` Can be method reference or literal value. [^json2]
+/// * `example = ...` Can be literal value, method reference or _`json!(...)`_. [^json2]
+/// * `default = ...` Can be literal value, method reference or _`json!(...)`_. [^json2]
 /// * `format = ...` [`ComponentFormat`][format] to use for the property. By default the format is derived from
 ///   the type of the property according OpenApi spec.
 /// * `write_only` Defines property is only used in **write** operations *POST,PUT,PATCH* but not in *GET*
@@ -103,6 +105,40 @@ use ext::ArgumentResolver;
 ///
 /// See [`Xml`][xml] for more details.
 ///
+/// # Partial `#[serde(...)]` attributes support
+///
+/// Component derive has partial support for [serde attributes](https://serde.rs/attributes.html). These supported attributes will reflect to the
+/// generated OpenAPI doc. For example if _`#[serde(skip)]`_ is defined the attribute will not show up in the OpenAPI spec at all since it will not never
+/// be serialized anyway. Similarly the _`rename`_ and _`rename_all`_ will reflect to the generated OpenAPI doc.
+///
+/// * `rename_all = "..."` Supported in container level.
+/// * `rename = "..."` Supported **only** in field or variant level.
+/// * `skip = "..."` Supported  **only** in field or variant level.
+///
+/// Other _`serde`_ attributes works as is but does not have any effect on the generated OpenAPI doc.
+///
+/// ```rust
+/// # use serde::Serialize;
+/// # use utoipa::Component;
+/// #[derive(Serialize, Component)]
+/// struct Foo(String);
+///
+/// #[derive(Serialize, Component)]
+/// #[serde(rename_all = "camelCase")]
+/// enum Bar {
+///     UnitValue,
+///     #[serde(rename_all = "camelCase")]
+///     NamedFields {
+///         #[serde(rename = "id")]
+///         named_id: &'static str,
+///         name_list: Option<Vec<String>>
+///     },
+///     UnnamedFields(Foo),
+///     #[serde(skip)]
+///     SkipMe,
+/// }
+/// ```
+///
 /// # Examples
 ///
 /// Example struct with struct level example.
@@ -202,7 +238,7 @@ use ext::ArgumentResolver;
 /// }
 /// ```
 ///
-/// Enforce type being used in OpenAPI spec to String with `value_type` and set format to octect stream
+/// Enforce type being used in OpenAPI spec to [`String`] with `value_type` and set format to octect stream
 /// with [`ComponentFormat::Binary`][binary].
 /// ```rust
 /// # use utoipa::Component;
@@ -214,7 +250,7 @@ use ext::ArgumentResolver;
 /// }
 /// ```
 ///
-/// Enforce type being used in OpenAPI spec to String with `value_type` option.
+/// Enforce type being used in OpenAPI spec to [`String`] with `value_type` option.
 /// ```rust
 /// # use utoipa::Component;
 /// #[derive(Component)]
@@ -274,10 +310,6 @@ pub fn derive_component(input: TokenStream) -> TokenStream {
 /// * `params(...)` Slice of params that the endpoint accepts.
 /// * `security(...)` List of [`SecurityRequirement`][security]s local to the path operation.
 ///
-/// > **Note!** when **actix_extras** feature is enabled the **operation**, **path** and **params** declaration
-/// > may be omitted since they are resolved from **actix-web** attributes namely **path** and function arguments.
-/// > To define description or other parameter info then **params** still need to be defined manually. See the example
-/// > in [examples section](#examples).
 ///
 /// # Request Body Attributes
 ///
@@ -360,12 +392,18 @@ pub fn derive_component(input: TokenStream) -> TokenStream {
 ///   E.g. _`path, query, header, cookie`_
 /// * `deprecated` Define whether the parameter is deprecated or not.
 /// * `description = "..."` Define possible description for the parameter as str.
+/// * `style = ...` Defines how parameters are serialized by [`ParameterStyle`][style]. Default values are based on _`in`_ attribute.
+/// * `explode` Defines whether new _`parameter=value`_ is created for each parameter withing _`object`_ or _`array`_.
+/// * `allow_reserved` Defines whether reserved charachers _`:/?#[]@!$&'()*+,;=`_ is allowed within value.
+/// * `example = ...` Can be literal value, method reference or _`json!(...)`_. [^json]. Given example
+///   will override any example in underlying parameter type.
 ///
 /// **Params supports following representation formats:**
 ///
 /// ```text
 /// ("id" = String, path, deprecated, description = "Pet database id"),
 /// ("id", path, deprecated, description = "Pet database id"),
+/// ("value" = Option<[String]>, query, description = "Value description", style = Form, allow_reserved, deprecated, explode, example = json!(["Value"]))
 /// ```
 ///
 /// # Security Requirement Attributes
@@ -388,11 +426,12 @@ pub fn derive_component(input: TokenStream) -> TokenStream {
 ///
 /// # actix_extras support for actix-web
 ///
-/// **actix_extas** feature gives **utoipa** ability to use **actix-web** path operation macros such as `#[get(...)]` to
-/// resolve path for `#[utoipa::path]`. Also it is able to parse the the `path` parameters with types from function arguments
-/// of operation which are defined within type `web::Path<...>`. Allowing you leave out types of parameters in `params(...)`
-/// section of even leave out the section if description is not needed for parameters. Utoipa is only able to resolve
-/// [primitive types][primitive] and [`String`] type. Using other types is undefined behaviour.
+/// **actix_extras** featue gives **utoipa** ability to parse path operation information from **actix-web** types and macros.
+///
+/// 1. Ability to parse `path` from **actix-web** path attribute macros e.g. _`#[get(...)]`_.
+/// 2. Ability to parse [`std::primitive`]  or [`String`] or [`tuple`] typed `path` parameters from **actix-web** _`web::Path<...>`_.
+/// 3. Ability to parse `path` and `query` parameters form **actix-web** _`web::Path<...>`_, _`web::Query<...>`_ types
+///    with [`IntoParams`][into_params] trait.
 ///
 /// See the **actix_extras** in action in examples [todo-actix](https://github.com/juhaku/utoipa/tree/master/examples/todo-actix).
 ///
@@ -436,7 +475,7 @@ pub fn derive_component(input: TokenStream) -> TokenStream {
 ///
 /// # rocket_extras support for rocket
 ///
-/// **rocket_extras** feature give **utoipa** ability to use **rocket** path operation macros such as `#[get(...)]` to
+/// **rocket_extras** feature gives **utoipa** ability to use **rocket** path operation macros such as _`#[get(...)]`_ to
 /// resolve path for `#[utoipa::path]`.  Also it is able to parse the `path` and `query` parameters from path operation macro
 /// combined with function arguments of the operation. Allowing you leave out types from parameters in `params(...)` section
 /// or even leave out the section if description is not needed for parameters. Utoipa is only able to parse parameter types
@@ -554,6 +593,8 @@ pub fn derive_component(input: TokenStream) -> TokenStream {
 /// [security]: openapi/security/struct.SecurityRequirement.html
 /// [security_schema]: openapi/security/struct.SecuritySchema.html
 /// [primitive]: https://doc.rust-lang.org/std/primitive/index.html
+/// [into_params]: trait.IntoParams.html
+/// [style]: openapi/path/enum.ParameterStyle.html
 ///
 /// [^json]: **json** feature need to be enabled for `json!(...)` type to work.
 ///
@@ -569,13 +610,16 @@ pub fn path(attr: TokenStream, item: TokenStream) -> TokenStream {
 
     let mut resolved_operation = PathOperations::resolve_operation(&ast_fn);
 
-    let mut resolved_path = PathOperations::resolve_path(
+    let resolved_path = PathOperations::resolve_path(
         &resolved_operation
             .as_mut()
             .map(|operation| mem::take(&mut operation.path))
             .or_else(|| path_attribute.path.as_ref().map(String::to_string)), // cannot use mem take because we need this later
     );
 
+    #[cfg(any(feature = "actix_extras", feature = "rocket_extras"))]
+    let mut resolved_path = resolved_path;
+
     #[cfg(any(feature = "actix_extras", feature = "rocket_extras"))]
     {
         let args = resolved_path.as_mut().map(|path| mem::take(&mut path.args));
@@ -699,10 +743,99 @@ pub fn openapi(input: TokenStream) -> TokenStream {
     );
 
     let openapi = OpenApi(openapi_attributes, ident);
-    quote! {
-        #openapi
-    }
-    .into()
+
+    openapi.to_token_stream().into()
+}
+
+#[cfg(feature = "actix_extras")]
+#[proc_macro_error]
+#[proc_macro_derive(IntoParams, attributes(param))]
+/// IntoParams derive macro for **actix-web** only.
+///
+/// This is `#[derive]` implementation for [`IntoParams`][into_params] trait.
+///
+/// Typically path parameters need to be defined within [`#[utoipa::path(...params(...))]`][path_params] section
+/// for the endpoint. But this trait eliminates the need for that when [`struct`][struct]s are used to define parameters.
+/// Still [`std::primitive`] and [`String`] path parameters or [`tuple`] style path parameters need to be defined
+/// within `params(...)` section if description or other than default configuration need to be given.
+///
+/// You can use the Rust's own `#[deprecated]` attribute on field to mark it as
+/// deprecated and it will reflect to the generated OpenAPI spec.
+///
+/// `#[deprecated]` attribute supports adding addtional details such as a reason and or since version
+/// but this is is not supported in OpenAPI. OpenAPI has only a boolean flag to determine deprecation.
+/// While it is totally okay to declare deprecated with reason
+/// `#[deprecated  = "There is better way to do this"]` the reason would not render in OpenAPI spec.
+///
+/// # IntoParams Attributes for `#[param(...)]`
+///
+/// * `style = ...` Defines how parameters are serialized by [`ParameterStyle`][style]. Default values are based on _`in`_ attribute.
+/// * `explode` Defines whether new _`parameter=value`_ is created for each parameter withing _`object`_ or _`array`_.
+/// * `allow_reserved` Defines whether reserved charachers _`:/?#[]@!$&'()*+,;=`_ is allowed within value.
+/// * `example = ...` Can be literal value, method reference or _`json!(...)`_. [^json] Given example
+///   will override any example in underlying parameter type.
+///
+/// # Examples
+///
+/// Demonstrate [`IntoParams`][into_params] usage with resolving `path` and `query` parameters
+/// for `get_pet` endpoint. [^actix]
+/// ```rust
+/// use actix_web::{get, HttpResponse, Responder};
+/// use actix_web::web::{Path, Query};
+/// use serde::Deserialize;
+/// use serde_json::json;
+/// use utoipa::IntoParams;
+///
+/// #[derive(Deserialize, IntoParams)]
+/// struct PetPathArgs {
+///     /// Id of pet
+///     id: i64,
+///     /// Name of pet
+///     name: String,
+/// }
+///
+/// #[derive(Deserialize, IntoParams)]
+/// struct Filter {
+///     /// Age filter for pets
+///     #[deprecated]
+///     #[param(style = Form, explode, allow_reserved, example = json!([10]))]
+///     age: Option<Vec<i32>>,
+/// }
+///
+/// #[utoipa::path(
+///     responses(
+///         (status = 200, description = "success response")
+///     )
+/// )]
+/// #[get("/pet/{id}/{name}")]
+/// async fn get_pet(pet: Path<PetPathArgs>, query: Query<Filter>) -> impl Responder {
+///     HttpResponse::Ok().json(json!({ "id": pet.id }))
+/// }
+/// ```
+///
+/// [into_params]: trait.IntoParams.html
+/// [path_params]: attr.path.html#params-attributes
+/// [struct]: https://doc.rust-lang.org/std/keyword.struct.html
+/// [style]: openapi/path/enum.ParameterStyle.html
+///
+/// [^actix]: Feature **actix_extras** need to be enabled
+///
+/// [^json]: **json** feature need to be enabled for `json!(...)` type to work.
+pub fn into_params(input: TokenStream) -> TokenStream {
+    let DeriveInput {
+        ident,
+        generics,
+        data,
+        ..
+    } = syn::parse_macro_input!(input);
+
+    let into_params = IntoParams {
+        generics,
+        data,
+        ident,
+    };
+
+    into_params.to_token_stream().into()
 }
 
 /// Tokenizes slice or Vec of tokenizable items as array either with reference (`&[...]`)
@@ -726,6 +859,19 @@ where
     }
 }
 
+impl<T> Deref for Array<T>
+where
+    T: Sized + ToTokens,
+{
+    type Target = Vec<T>;
+
+    fn deref(&self) -> &Self::Target {
+        match self {
+            Self::Owned(vec) => vec,
+        }
+    }
+}
+
 impl<T> ToTokens for Array<T>
 where
     T: Sized + ToTokens,
@@ -773,6 +919,7 @@ impl ToTokens for Deprecated {
     }
 }
 
+#[cfg_attr(feature = "debug", derive(Debug))]
 enum Required {
     True,
     False,
@@ -921,19 +1068,101 @@ impl ToTokens for ExternalDocs {
     }
 }
 
+/// Represents OpenAPI Any value used in example and defualt fields.
 #[cfg_attr(feature = "debug", derive(Debug))]
-enum Example {
+pub(self) enum AnyValue {
     String(TokenStream2),
     Json(TokenStream2),
+    #[cfg(not(feature = "json"))]
+    Literal(TokenStream2),
 }
 
-impl ToTokens for Example {
+impl AnyValue {
+    fn parse_any(input: ParseStream) -> syn::Result<Self> {
+        if input.peek(Lit) {
+            if input.peek(LitStr) {
+                let lit_str = input.parse::<LitStr>().unwrap().to_token_stream();
+
+                #[cfg(feature = "json")]
+                {
+                    Ok(AnyValue::Json(lit_str))
+                }
+                #[cfg(not(feature = "json"))]
+                {
+                    Ok(AnyValue::String(lit_str))
+                }
+            } else {
+                let lit = input.parse::<Lit>().unwrap().to_token_stream();
+
+                #[cfg(feature = "json")]
+                {
+                    Ok(AnyValue::Json(lit))
+                }
+                #[cfg(not(feature = "json"))]
+                {
+                    Ok(AnyValue::Literal(lit))
+                }
+            }
+        } else {
+            let fork = input.fork();
+            let is_json = if fork.peek(syn::Ident) && fork.peek2(Token![!]) {
+                let ident = fork.parse::<Ident>().unwrap();
+                ident == "json"
+            } else {
+                false
+            };
+
+            if is_json {
+                let json = parse_utils::parse_json_token_stream(input)?;
+
+                #[cfg(feature = "json")]
+                {
+                    Ok(AnyValue::Json(json))
+                }
+                #[cfg(not(feature = "json"))]
+                {
+                    Ok(AnyValue::Literal(json))
+                }
+            } else {
+                let method = input.parse::<ExprPath>().map_err(|error| {
+                    syn::Error::new(
+                        error.span(),
+                        "expected literal value, json!(...) or method reference",
+                    )
+                })?;
+
+                #[cfg(feature = "json")]
+                {
+                    Ok(AnyValue::Json(quote! { #method() }))
+                }
+                #[cfg(not(feature = "json"))]
+                {
+                    Ok(AnyValue::Literal(quote! { #method() }))
+                }
+            }
+        }
+    }
+
+    fn parse_lit_str_or_json(input: ParseStream) -> syn::Result<Self> {
+        if input.peek(LitStr) {
+            Ok(AnyValue::String(
+                input.parse::<LitStr>().unwrap().to_token_stream(),
+            ))
+        } else {
+            Ok(AnyValue::Json(parse_utils::parse_json_token_stream(input)?))
+        }
+    }
+}
+
+impl ToTokens for AnyValue {
     fn to_tokens(&self, tokens: &mut TokenStream2) {
         match self {
             Self::Json(json) => tokens.extend(quote! {
                 serde_json::json!(#json)
             }),
             Self::String(string) => tokens.extend(string.to_owned()),
+            #[cfg(not(feature = "json"))]
+            Self::Literal(literal) => tokens.extend(quote! { format!("{}", #literal) }),
         }
     }
 }
@@ -941,8 +1170,7 @@ impl ToTokens for Example {
 /// Parsing utils
 mod parse_utils {
     use proc_macro2::{Group, Ident, TokenStream};
-    use proc_macro_error::{abort, ResultExt};
-    use quote::ToTokens;
+    use proc_macro_error::ResultExt;
     use syn::{
         parenthesized,
         parse::{Parse, ParseStream},
@@ -951,8 +1179,6 @@ mod parse_utils {
         Error, LitBool, LitStr, Token,
     };
 
-    use crate::Example;
-
     pub fn parse_next<T: Sized>(input: ParseStream, next: impl FnOnce() -> T) -> T {
         input
             .parse::<Token![=]>()
@@ -1018,28 +1244,4 @@ mod parse_utils {
             ))
         }
     }
-
-    fn parse_next_lit_str_or_json(input: ParseStream, abort_op: impl FnOnce(&Error)) -> Example {
-        if input.peek2(LitStr) {
-            Example::String(parse_next(input, || {
-                input.parse::<LitStr>().unwrap().to_token_stream()
-            }))
-        } else {
-            Example::Json(parse_next(input, || {
-                parse_json_token_stream(input).unwrap_or_else(|error| {
-                    abort_op(&error);
-                    // hacky way to tell rust that we are having a "never" type here
-                    unreachable!("oops! unreachable code we should have aborted here");
-                })
-            }))
-        }
-    }
-
-    pub(crate) fn parse_next_lit_str_or_json_example(input: ParseStream, ident: &Ident) -> Example {
-        parse_next_lit_str_or_json(input, |error| {
-            abort! {ident, "unparseable example, expected json!(), {}", error;
-            help = r#"Try defining example = json!({{"key": "value"}})"#;
-            }
-        })
-    }
 }
